﻿/*********************************************************************
 * The MIT License (MIT)
 * 
 * Copyright (c) 2010 Manish Sinha <mail@manishsinha.net>
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:

 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 ********************************************************************/


namespace LpNet.WadlSharp.Common.Generator
{
    using System.CodeDom;
    using System;
    using Interface.Generator;
    using System.Collections.Generic;

    public class LpConstructor : IConstructor
    {
        #region IConstructor Members

        public CodeConstructor Create(bool uri, bool httpHeaders, bool customizeInterface, CodeTypeDeclaration typeDeclaration, HashSet<string> nameSpaces, CodeNamespace ns)
        {
            CodeConstructor constructor = new CodeConstructor();

            constructor.Name = typeDeclaration.Name;
            constructor.Attributes = MemberAttributes.Public;

            #region Creating the Parameters, the private fields and assignments

            if (uri)
            {
                Type uriType = typeof (string);

                #region Creating the input url parameter

                CodeParameterDeclarationExpression uriParam = new CodeParameterDeclarationExpression();
                uriParam.Type = new CodeTypeReference(uriType);
                uriParam.Name = IdentifierNames.ResourceUri;
                constructor.Parameters.Add(uriParam);

                #endregion

                #region Create the Field

                string idFieldName = string.Format("_{0}", uriParam.Name);
                IField fld = new LpField();
                CodeMemberField memberField = fld.Create(idFieldName, uriType, nameSpaces);
                typeDeclaration.Members.Add(memberField);

                #endregion

                #region Assignment of arguments to private field

                constructor.Statements.Add(
                                            new CodeAssignStatement(
                                                                    new CodeFieldReferenceExpression(
                                                                                                    new CodeThisReferenceExpression(),
                                                                                                    idFieldName
                                                                                                    ),
                                                                    new CodeVariableReferenceExpression(uriParam.Name)
                                                                    )
                                           );

                #endregion
            }

            if (httpHeaders)
            {
                string headersType = "Dictionary<string,string>";

                #region Creating the input url parameter

                CodeParameterDeclarationExpression httpHeadersParam = new CodeParameterDeclarationExpression();
                httpHeadersParam.Type = new CodeTypeReference(headersType);
                httpHeadersParam.Name = IdentifierNames.HttpHeaders;
                constructor.Parameters.Add(httpHeadersParam);

                #endregion

                #region Create the Field

                string httpHeadersFieldName = string.Format("_{0}", httpHeadersParam.Name);
                IField fld = new LpField();
                CodeMemberField memberField = fld.Create(httpHeadersFieldName, headersType, nameSpaces);
                typeDeclaration.Members.Add(memberField);

                #endregion

                #region Assignment of arguments to private field

                constructor.Statements.Add(
                                            new CodeAssignStatement(
                                                                    new CodeFieldReferenceExpression(
                                                                                                    new CodeThisReferenceExpression(),
                                                                                                    httpHeadersFieldName
                                                                                                    ),
                                                                    new CodeVariableReferenceExpression(httpHeadersParam.Name)
                                                                    )
                                           );

                #endregion
            }

            if (customizeInterface)
            {
                string customizeInterfaceType = "ICustomize";

                #region Creating the input url parameter

                CodeParameterDeclarationExpression customizeInterfaceParam = new CodeParameterDeclarationExpression();
                customizeInterfaceParam.Type = new CodeTypeReference(customizeInterfaceType);
                customizeInterfaceParam.Name = IdentifierNames.CustomizeInterface;
                constructor.Parameters.Add(customizeInterfaceParam);

                #endregion

                #region Create the Field

                string customizeInterfaceFieldName = string.Format("_{0}", customizeInterfaceParam.Name);
                IField fld = new LpField();
                CodeMemberField memberField = fld.Create(customizeInterfaceFieldName, customizeInterfaceType, nameSpaces);
                typeDeclaration.Members.Add(memberField);

                #endregion

                #region Assignment of arguments to private field

                constructor.Statements.Add(
                                            new CodeAssignStatement(
                                                                    new CodeFieldReferenceExpression(
                                                                                                    new CodeThisReferenceExpression(),
                                                                                                    customizeInterfaceFieldName
                                                                                                    ),
                                                                    new CodeVariableReferenceExpression(customizeInterfaceParam.Name)
                                                                    )
                                           );

                #endregion
            }

            #endregion

            return constructor;
        }

        #endregion
    }
}
