﻿/*********************************************************************
 * The MIT License (MIT)
 * 
 * Copyright (c) 2010 Manish Sinha <mail@manishsinha.net>
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:

 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 ********************************************************************/


namespace LpNet.WadlSharp.Common.Generator
{
    using System.CodeDom;
    using Interface.Generator;
    using Resources;
    using System.Collections.Generic;
    using System.Text.RegularExpressions;

    public class LpEnum : IEnum
    {
        #region IEnum Members

        public CodeTypeDeclaration Create(string enumName, option[] discreteValues, HashSet<string> nameSpaces)
        {
            CodeTypeDeclaration enumCandidate = new CodeTypeDeclaration();

            enumCandidate.Name = enumName;
            enumCandidate.IsEnum = true;
            enumCandidate.Attributes = MemberAttributes.Public | MemberAttributes.Static;

            foreach (option discreteValue in discreteValues)
            {
                string value = discreteValue.value;
                value = Regex.Replace(value, @"\s+", "_");
                value = Regex.Replace(value, @"[^a-zA-Z0-9_]", "");

                CodeMemberField field = new CodeMemberField(enumName, value);
                field.CustomAttributes.Add(new CodeAttributeDeclaration("Description",
                                                                        new CodeAttributeArgument(
                                                                            new CodePrimitiveExpression(
                                                                                discreteValue.value))));

                enumCandidate.Members.Add(field);
            }

            return enumCandidate;
        }

        #endregion
    }
}
