﻿/*********************************************************************
 * The MIT License (MIT)
 * 
 * Copyright (c) 2010 Manish Sinha <mail@manishsinha.net>
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:

 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 ********************************************************************/


namespace LpNet.WadlSharp.Common.Generator
{
    using System.CodeDom;
    using System.Collections.Generic;
    using Interface.Generator;
    using Resources;
    using Enum;

    public class LpMethod : IMethod
    {
        #region Public Methods Exposed by Interface

        public CodeMemberMethod Create(method methodInstance, HashSet<string> nameSpaces, CodeNamespace ns)
        {
            // Create am instance of CodeMemberMethod and set it's name
            CodeMemberMethod memberMethod = new CodeMemberMethod();
            memberMethod.Name = WadlSharpUtils.CreateIdentifier(methodInstance.id);
            memberMethod.Attributes = MemberAttributes.Public | MemberAttributes.Final;

            #region Create and initialize Dictionary<string,string>

            DefineKeyValueDictionary(memberMethod);

            #endregion

            #region Add the Parameters using param type

            if (methodInstance.request != null && methodInstance.request.param != null)
            {
                param[] listOfParam = methodInstance.request.param;

                // For each parameter
                CreateInputParameters(memberMethod, nameSpaces, ns, listOfParam, methodInstance);
            }


            #endregion

            #region Call Serialze method in the interface

            CallSerializeMethod(memberMethod, methodInstance);

            #endregion

            #region Call Fetch method in the interface

            CallFetchMethod(memberMethod, methodInstance);

            #endregion

            #region Add a return statement

            if (methodInstance.response != null)
            {
                #region Create Method summary comment and Return Type

                CreateCommentAndReturn(memberMethod, methodInstance, nameSpaces, ns);

                #endregion

                if (string.Equals(memberMethod.ReturnType.BaseType, "System.Void") == false)
                {
                    #region Call Parse method in the interface

                    CallParseMethod(memberMethod);

                    #endregion

                    CreateMethodRetunStatement(memberMethod);
                }
            }

            #endregion

            return memberMethod;
        }

        public CodeMemberMethod Create(method methodInstance, representation_type paramSet, HashSet<string> nameSpaces, CodeNamespace ns)
        {
            if (paramSet != null)
            {
                // Create am instance of CodeMemberMethod and set it's name
                CodeMemberMethod memberMethod = new CodeMemberMethod();
                memberMethod.Name = WadlSharpUtils.CreateIdentifier(methodInstance.id);
                memberMethod.Attributes = MemberAttributes.Public | MemberAttributes.Final;

                #region Create and initialize Dictionary<string,string>

                DefineKeyValueDictionary(memberMethod);

                #endregion

                #region Add the Input parameters

                if (paramSet.param != null)
                {
                    CreateInputParameters(memberMethod, nameSpaces, ns, paramSet.param, methodInstance);
                }
                else if (paramSet.href != null)
                {
                    CreateInputParameters(memberMethod, nameSpaces, ns, paramSet, methodInstance);
                }

                #endregion

                #region Call Serialze method in the interface

                CallSerializeMethod(memberMethod, methodInstance);

                #endregion

                #region Call Fetch method in the interface

                CallFetchMethod(memberMethod, methodInstance);

                #endregion

                #region Add a return statement

                if (methodInstance.response != null)
                {
                    #region Create Method summary comment and Return Type

                    CreateCommentAndReturn(memberMethod, methodInstance, nameSpaces, ns);

                    #endregion

                    if (string.Equals(memberMethod.ReturnType.BaseType, "System.Void") == false)
                    {
                        #region Call Parse method in the interface

                        CallParseMethod(memberMethod);

                        #endregion

                        CreateMethodRetunStatement(memberMethod);
                    }
                }

                #endregion

                return memberMethod;
            }

            return null;
        }

        #endregion

        private void CreateInputParameters(CodeMemberMethod memberMethod, HashSet<string> nameSpaces, CodeNamespace ns, param[] listOfParam, method originalMethod)
        {
            foreach (param param in listOfParam)
            {
                #region Create the Parameter

                CodeParameterDeclarationExpression parameterDeclarationExpression;

                string paramIden = WadlSharpUtils.CreateIdentifier(param.name);

                if (param.@fixed == null)
                {
                    // If the parameter is just a normal type
                    if (param.option == null)
                    {
                        parameterDeclarationExpression = new CodeParameterDeclarationExpression(typeof (string),
                                                                                                paramIden);

                        #region Add the parameters to the Dictionary

                        CodeMethodInvokeExpression addParam = new CodeMethodInvokeExpression(
                                                                    new CodeMethodReferenceExpression(
                                                                                                    new CodeFieldReferenceExpression(
                                                                                                                                null,
                                                                                                                                keyValueField
                                                                                                                                ),
                                                                                                    "Add"
                                                                                                      ),
                                                                    new CodeExpression[]
                                                                        {
                                                                            new CodePrimitiveExpression(param.name),
                                                                            new CodeFieldReferenceExpression(null,paramIden)
                                                                        }
                                                                                        );
                        memberMethod.Statements.Add(addParam);

                        #endregion
                    }
                        // If the parameter is an option/enumertion
                    else
                    {
                        #region Checking if the Enumeration already exists

                        bool enumExists = false;
                        foreach (CodeTypeDeclaration type in ns.Types)
                        {
                            if (type.IsEnum)
                            {
                                if (string.Equals(type.Name, param.name))
                                {
                                    enumExists = true;
                                    continue;
                                }

                            }
                        }

                        if (enumExists)
                        {
                            continue;
                        }

                        #endregion

                        // Create the enumeration
                        IEnum enumeration = new LpEnum();
                        CodeTypeDeclaration customEnum = enumeration.Create(paramIden, param.option, nameSpaces);

                        // Add the custom Created Type in the list of Types of this namespace
                        ns.Types.Add(customEnum);

                        string paramType = param.name;
                        string enumFieldName = string.Format("{0}_param", param.name);
                        parameterDeclarationExpression = new CodeParameterDeclarationExpression(paramType, enumFieldName);
                    }

                    // Add the newly created parameter
                    memberMethod.Parameters.Add(parameterDeclarationExpression);
                }
                    // For parameters with fixed values
                else
                {
                    #region Add the parameters to the Dictionary

                    CodeMethodInvokeExpression addParam = new CodeMethodInvokeExpression(
                                                                    new CodeMethodReferenceExpression(
                                                                                                    new CodeFieldReferenceExpression(
                                                                                                                                null,
                                                                                                                                keyValueField
                                                                                                                                ),
                                                                                                    "Add"
                                                                                                      ),
                                                                    new CodeExpression[]
                                                                        {
                                                                            new CodePrimitiveExpression(param.name),
                                                                            new CodePrimitiveExpression(param.@fixed)
                                                                        }
                                                                                        );
                    memberMethod.Statements.Add(addParam);

                    #endregion
                }

                #endregion

                #region Create the entry for the parameter's in Method's XML Comments

                if (param.doc != null)
                {
                    foreach (doc docInst in param.doc)
                    {
                        IComment comm = new LpComment();
                        memberMethod.Comments.AddRange(
                            comm.Create(
                                docInst,
                                LanguageConstructType.InputParameter,
                                param.name
                                )
                            );
                    }
                }

                #endregion
            }
        }

        private void CreateInputParameters(CodeMemberMethod memberMethod, HashSet<string> nameSpaces, CodeNamespace ns, representation_type paramSet, method originalMethod)
        {
            CodeParameterDeclarationExpression parameterDeclarationExpression;

            string paramType = WadlSharpUtils.CreateAnchor(paramSet.href);
            paramType = WadlSharpUtils.CreateIdentifier(paramType);
            string paramName = string.Format("{0}_field", paramType);

            parameterDeclarationExpression = new CodeParameterDeclarationExpression(paramType, paramName);

            memberMethod.Parameters.Add(parameterDeclarationExpression);

            #region Create the entry for the parameter's in Method's XML Comments

            if (paramSet.doc != null)
            {
                foreach (doc docInst in paramSet.doc)
                {
                    IComment comm = new LpComment();
                    memberMethod.Comments.AddRange(
                        comm.Create(
                            docInst,
                            LanguageConstructType.InputParameter,
                            paramName
                            )
                        );
                }
            }

            #endregion
        }

        private void CreateCommentAndReturn(CodeMemberMethod memberMethod, method methodInstance, HashSet<string> nameSpaces, CodeNamespace ns)
        {
            #region  Method Name Comment

            if (methodInstance.doc != null)
            {
                IComment methodName = new LpComment();

                foreach (doc documentation in methodInstance.doc)
                {
                    CodeCommentStatementCollection coll = methodName.Create(documentation, LanguageConstructType.Property, methodInstance.name);

                    memberMethod.Comments.AddRange(coll);
                }
            }

            #endregion

            #region Return Type

            if (methodInstance.response != null)
            {
                if (methodInstance.response.Items != null)
                {
                    foreach (representation_type s in methodInstance.response.Items)
                    {
                        #region Set the Return Type

                        if (s.href != null)
                        {
                            IReturn ret = new LpReturn();

                            memberMethod.ReturnType.BaseType = ret.Create(s, nameSpaces, ns);
                        }

                        #endregion

                        #region Create the entry for the parameter's in Method's XML Comments

                        if (s.doc != null)
                        {
                            foreach (doc docInst in s.doc)
                            {
                                IComment comm = new LpComment();
                                memberMethod.Comments.AddRange(
                                    comm.Create(
                                        docInst,
                                        LanguageConstructType.InputParameter,
                                        null
                                        )
                                    );
                            }
                        }

                        #endregion
                    }
                }
                else if (methodInstance.response.param != null)
                {
                    foreach (param param in methodInstance.response.param)
                    {
                        if(param.style == ParamStyle.header)
                        {
                            memberMethod.ReturnType.BaseType = WadlSharpUtils.CreateAnchor(param.link.resource_type);
                            break;
                        }
                    }
                }
            }

            #endregion
        }

        #region Helper Methods

        private void DefineKeyValueDictionary(CodeMemberMethod memberMethod)
        {
            CodeVariableDeclarationStatement declareKeyValueHash = new CodeVariableDeclarationStatement(
                "Dictionary<string,string>",
                keyValueField,
                new CodeObjectCreateExpression("Dictionary<string, string>", new CodeExpression[] { })
                );
            memberMethod.Statements.Add(declareKeyValueHash);
        }

        private void CallSerializeMethod(CodeMemberMethod memberMethod, method methodInstance)
        {
            CodeMethodInvokeExpression invokeExpression = new CodeMethodInvokeExpression(
                new CodeMethodReferenceExpression(
                    new CodeFieldReferenceExpression(
                        null,
                        string.Format("_{0}", IdentifierNames.CustomizeInterface)
                        ),
                    IdentifierNames.SerializeMethodName
                    ),
                new CodeExpression[]
                    {
                        new CodeFieldReferenceExpression(new CodeFieldReferenceExpression(null,IdentifierNames.HttpMethodTypeEnum),methodInstance.name),
                        new CodeFieldReferenceExpression(null,keyValueField)
                    }
                );
            CodeVariableDeclarationStatement serializeData = new CodeVariableDeclarationStatement(
                typeof(string),
                serializedDataName,
                invokeExpression
                );

            memberMethod.Statements.Add(serializeData);
        }

        private void CallFetchMethod(CodeMemberMethod memberMethod, method methodInstance)
        {
            CodeMethodInvokeExpression fetchInvokeExpression = new CodeMethodInvokeExpression(
                new CodeMethodReferenceExpression(
                    new CodeFieldReferenceExpression(
                        null,
                        string.Format("_{0}", IdentifierNames.CustomizeInterface)
                        ),
                    IdentifierNames.FetchMethodName
                    ),
                new CodeExpression[]
                    {
                        new CodeFieldReferenceExpression(new CodeFieldReferenceExpression(null,IdentifierNames.HttpMethodTypeEnum),methodInstance.name),
                        new CodeFieldReferenceExpression(new CodeThisReferenceExpression(),string.Format("_{0}",IdentifierNames.ResourceUri)),
                        new CodeFieldReferenceExpression(new CodeThisReferenceExpression(),string.Format("_{0}",IdentifierNames.HttpHeaders)),
                        new CodeFieldReferenceExpression(null,serializedDataName)
                    }
                );
            CodeVariableDeclarationStatement fetchedData = new CodeVariableDeclarationStatement(
                typeof(string),
                fetchedDataName,
                fetchInvokeExpression
                );

            memberMethod.Statements.Add(fetchedData);
        }

        private void CallParseMethod(CodeMemberMethod memberMethod)
        {
            CodeMethodInvokeExpression parseInvokeExpression = new CodeMethodInvokeExpression(
                new CodeMethodReferenceExpression(
                    new CodeVariableReferenceExpression(string.Format("_{0}", IdentifierNames.CustomizeInterface)),
                    IdentifierNames.ParseMethodName,
                    new CodeTypeReference[]
                        {
                            new CodeTypeReference(memberMethod.ReturnType.BaseType)
                        }
                    ),
                new CodeExpression[]
                    {
                        new CodeFieldReferenceExpression(null, fetchedDataName)
                    }
                );
            CodeVariableDeclarationStatement parsedData = new CodeVariableDeclarationStatement(
                memberMethod.ReturnType.BaseType,
                parsedDataName,
                parseInvokeExpression
                );

            memberMethod.Statements.Add(parsedData);
        }

        private void CreateMethodRetunStatement(CodeMemberMethod memberMethod)
        {
            CodeMethodReturnStatement returnStatement = new CodeMethodReturnStatement(new CodeArgumentReferenceExpression(parsedDataName));
            memberMethod.Statements.Add(returnStatement);
        }

        #endregion

        #region Field Names

        string keyValueField = "keyValue";

        string parsedDataName = "parsedData";

        string serializedDataName = "serializedData";

        string fetchedDataName = "fetchedData";

        #endregion
    }
}
