﻿/*********************************************************************
 * The MIT License (MIT)
 * 
 * Copyright (c) 2010 Manish Sinha <mail@manishsinha.net>
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:

 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 ********************************************************************/

using LpNet.WadlSharp.Common.Custom;

namespace LpNet.WadlSharp.Common.Generator
{
    using System.CodeDom;
    using System.Collections.Generic;
    using Interface.Generator;
    using Resources;

    public class LpNamespace : INamespace
    {
        #region INamespace Members

        public CodeNamespace Create(application app, string namespaceName)
        {
            CodeNamespace ns = new CodeNamespace(namespaceName);
            HashSet<string> namespaces = new HashSet<string>();

            ResourceList.Resource = app.resources;

            // Check all the items
            foreach (object item in app.Items)
            {
                if (item.GetType() == typeof(representation_type))
                {
                    representation_type repType = item as representation_type;

                    IClass customType = new LpClass();
                    CodeTypeDeclaration typeDec = customType.Create(repType, namespaces, ns);

                    ns.Types.Add(typeDec);
                }
                else if (item.GetType() == typeof(resource_type))
                {
                    resource_type resType = item as resource_type;

                    IClass customType = new LpClass();
                    CodeTypeDeclaration typeDec = customType.Create(resType, namespaces, ns);
                    
                    ns.Types.Add(typeDec);
                }
            }

            CodeTypeDeclaration customizeInterface = Customizer.CreateICustomize();
            ns.Types.Add(customizeInterface);

            CodeTypeDeclaration httpMethodEnum = Customizer.CreateHttpMethodTypes();
            ns.Types.Add(httpMethodEnum);

            CodeTypeDeclaration enumDesc = Customizer.CreateEnumDescription();
            ns.Types.Add(enumDesc);

            return ns;
        }

        #endregion
    }
}
