﻿/*********************************************************************
 * The MIT License (MIT)
 * 
 * Copyright (c) 2010 Manish Sinha <mail@manishsinha.net>
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:

 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 ********************************************************************/


namespace LpNet.WadlSharp.Common.Generator
{
    using System.Collections.Generic;
    using Interface.Generator;
    using System.CodeDom;
    using Resources;

    public class LpParameter : IParameter
    {
        #region IParameter Members

        /// <summary>
        /// Creates parameter for method for supplied param instance
        /// </summary>
        /// <param name="param">The supplied param instance</param>
        /// <param name="nameSpaces">The list to which all required namespaces refernces should be added</param>
        /// <param name="ns">The root namespace for all the generated Types</param>
        /// <returns>The instance of parameter</returns>
        public CodeParameterDeclarationExpression Create(param param, HashSet<string> nameSpaces, CodeNamespace ns)
        {
            suppliedParam = param;

            CodeParameterDeclarationExpression paramExp = new CodeParameterDeclarationExpression();

            paramExp.Name = WadlSharpUtils.CreateIdentifier(param.name);
            paramExp.Type = new CodeTypeReference(typeof(string));

            return paramExp;
        }

        public param SuppliedParameter
        {
            get
            {
                return suppliedParam;
            }
        }

        #endregion

        #region Private Fields

        private param suppliedParam;

        #endregion
    }
}
