﻿/*********************************************************************
 * The MIT License (MIT)
 * 
 * Copyright (c) 2010 Manish Sinha <mail@manishsinha.net>
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:

 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 ********************************************************************/


namespace LpNet.WadlSharp.Common.Generator
{
    using System;
    using System.CodeDom;
    using Common;
    using Enum;
    using Interface.Generator;
    using Resources;
    using System.Collections.Generic;

    public class LpProperty : IProperty
    {
        #region IProperty Method

        public CodeMemberProperty Create(param propertyInstance, out CodeMemberField memberField, HashSet<string> nameSpaces, CodeNamespace ns)
        {
            #region Checking for null parameters and storing of values
            if (propertyInstance == null)
                throw new ArgumentNullException("propertyInstance");

            propertyField = propertyInstance;
            #endregion

            #region Create the CodeMemberProperty Instance and do all mappings

            CodeMemberProperty property = new CodeMemberProperty();

            Type dataType = typeof (string);

            property.Name = WadlSharpUtils.CreateIdentifier(propertyInstance.name);
            property.Attributes = MemberAttributes.Public | MemberAttributes.Final;
            property.Type = new CodeTypeReference(dataType);

            if (propertyInstance.link != null && propertyInstance.link.resource_type != null)
            {
                property.CustomAttributes.Add(new CodeAttributeDeclaration("Description",
                                                                           new CodeAttributeArgument(
                                                                               new CodePrimitiveExpression(
                                                                                   WadlSharpUtils.CreateAnchor(
                                                                                       propertyInstance.link.resource_type)))));
            }

            string fieldName = string.Format("_{0}", property.Name);
            property.GetStatements.Add(new CodeMethodReturnStatement(new CodeFieldReferenceExpression(new CodeThisReferenceExpression(), fieldName)));
            property.SetStatements.Add(new CodeAssignStatement(new CodeFieldReferenceExpression(new CodeThisReferenceExpression(), fieldName), new CodePropertySetValueReferenceExpression()));


            #region Create Property Documentation

            if (propertyInstance.doc != null)
            {
                IComment comm = new LpComment();

                #region Loop over all the doc instances and add them
                foreach (doc documentation in propertyInstance.doc)
                {
                    CodeCommentStatementCollection coll = comm.Create(documentation, LanguageConstructType.Property, propertyInstance.name);

                    property.Comments.AddRange(coll);
                }
                #endregion
            }

            #endregion

            #endregion

            #region Creating the Field

            IField fld = new LpField();
            memberField = fld.Create(fieldName, dataType, nameSpaces);

            // Save this in a private Field
            field = memberField;

            #endregion

            return property;
        }

        #endregion

        #region IProperty Properties

        /// <summary>
        /// The CodeMemberProperty representation of this property
        /// </summary>
        public CodeMemberProperty Property
        {
            get
            {
                return propertyMember;
            }
        }

        /// <summary>
        /// Whether this Property is mandatory or not
        /// </summary>
        public bool Required
        {
            get
            {
                return propertyField.required;
            }
        }

        /// <summary>
        /// The comments for this property
        /// </summary>
        public CodeCommentStatementCollection Comments
        {
            get
            {
                return propertyMember.Comments;
            }
        }

        public CodeMemberField Field
        {
            get
            {
                return field;
            }
        }

        #endregion

        #region Private Fields

        private param propertyField;

        private CodeMemberProperty propertyMember;

        public CodeMemberField field;

        #endregion
    }
}
