﻿/*********************************************************************
 * The MIT License (MIT)
 * 
 * Copyright (c) 2010 Manish Sinha <mail@manishsinha.net>
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:

 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 ********************************************************************/


namespace LpNet.WadlSharp.Common.Generator
{
    using System.Collections.Generic;
    using System.Data;
    using Interface.Generator;
    using Resources;
    using System.CodeDom;

    public class LpReturn : IReturn
    {
        #region IReturn Members

        /// <summary>
        /// Creates a return type out of the provided representation_type. Can throw ConstraintException
        /// </summary>
        /// <param name="repType">The representation for the return type</param>
        /// <param name="nameSpaces">The list to which all required namespaces refernces should be added</param>
        /// <returns>The string representation for type of the return value</returns>
        public string Create(representation_type repType, HashSet<string> nameSpaces, CodeNamespace ns)
        {
            int hashPos = repType.href.IndexOf('#');
            if (hashPos != -1)
            {
                string href = repType.href.Substring(hashPos + 1);
                href = WadlSharpUtils.CreateIdentifier(href);

                return href;
            }

            throw new ConstraintException("The href field of representation_type should be an anchor containing #");
        }

        #endregion
    }
}
