﻿/*********************************************************************
 * The MIT License (MIT)
 * 
 * Copyright (c) 2010 Manish Sinha <mail@manishsinha.net>
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:

 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 ********************************************************************/


namespace LpNet.WadlSharp.Common.Interface.Generator
{
    using System.CodeDom;
    using Resources;
    using System.Collections.Generic;

    public interface IProperty
    {
        /// <summary>
        /// Creates the Property from the param instance passed
        /// </summary>
        /// <param name="propertyInstance">The param instance from which property is to be created</param>
        /// <param name="memberField">This is an out parameter where the Field instance is saved</param>
        /// <param name="nameSpaces">The list of namespaces import required in the final generated code</param>
        /// <param name="ns">The root namespace for all the generated Types</param>
        /// <returns>A CodeMemberProperty instance</returns>
        CodeMemberProperty Create(param propertyInstance, out CodeMemberField memberField, HashSet<string> nameSpaces, CodeNamespace ns);

        /// <summary>
        /// The CodeMemberProperty instance created by the Create method
        /// </summary>
        CodeMemberProperty Property
        {
            get;
        }

        /// <summary>
        /// Whether this param instance is marked as Required or not
        /// </summary>
        bool Required
        {
            get;
        }

        /// <summary>
        /// The comments related to this Property
        /// </summary>
        CodeCommentStatementCollection Comments
        {
            get;
        }

        /// <summary>
        /// The corrosponding field created for this property
        /// </summary>
        CodeMemberField Field
        {
            get;
        }
    }
}
