﻿/*********************************************************************
 * The MIT License (MIT)
 * 
 * Copyright (c) 2010 Manish Sinha <mail@manishsinha.net>
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:

 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 ********************************************************************/


namespace LpNet.WadlSharp.Common
{
    using System.IO;
    using System.Xml.Serialization;
    using Interface;
    using Resources;

    /// <summary>
    /// Implements ILoader which is entrusted with loading a WADL file as per the schema
    /// </summary>
    public class WadlLoader : ILoader
    {
        #region ILoader Members

        /// <summary>
        /// Load the WADL file.
        /// Possible Exceptions: 
        /// 1) FileNotFoundException for non-existing file
        /// 2) IOException for invalid characters in the file path
        /// </summary>
        /// <param name="filePath">The WADL file path</param>
        /// <returns>An instance of application (WADL Root Node)</returns>
        public application Load(string filePath)
        {
            // Create an XML Serializer for the type - 'application'
            XmlSerializer serializer = new XmlSerializer(typeof(application));

            // Open the stream with the provided file
            StreamReader reader = new StreamReader(filePath);

            // Try to deserilaize and type-cast the resulting type to application
            application wadlInst = serializer.Deserialize(reader) as application;

            // Close the Stream
            reader.Close();

            return wadlInst;
        }

        #endregion
    }
}
